/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.enterprise.cli.commands;

import java.util.Iterator;
import java.util.Vector;
import com.sun.enterprise.cli.framework.*;

/**
 *  This class is the export command.
 *  It will set the given environment variable in multimode.
 */
public class ExportCommand extends S1ASCommand 
{

    private final static String ENVIRONMENT_PREFIX = "AS_ADMIN_";
    private final static String ENVIRONMENT_DELIMITER = "=";
    private final static String PASSWORD_STRING = "password";

    private CommandEnvironment commandEnvironment = CommandEnvironment.getInstance();

    /** Creates new ExportCommand */
    public ExportCommand() 
    {
    }

    /**
     * Validates the Options for correctness
     * @return boolean returns true if validation is succesful else false
     */
    public boolean validateOptions() throws CommandValidationException 
    {
        return super.validateOptions();
    }
    
    /**
     * Executes the command
     * @throws CommandException
     */
    public void runCommand() throws CommandException, CommandValidationException 
    {
        validateOptions();
        
        Vector operands = getOperands();
        if (operands.size() == 0)
        {
            printEnvValues();
            return;
        }
        for (int ii = 0; ii < operands.size(); ii++)
        {
            updateEnvironment((String)operands.elementAt(ii));
        }
    }

    
    /**
     * this method check for prefix AS_ADMIN
     * retursn the name without the prefix AS_ADMIN
     */
    private String checkForPrefix(String name) throws CommandException
    {
        String envName = null;
        //check for prefix AS_ADMIN
        if (name.regionMatches(true, 0, ENVIRONMENT_PREFIX, 0, 
                               ENVIRONMENT_PREFIX.length())) 
        {
	        //extract AS_ADMIN from sOperandName
            envName =  name.substring(ENVIRONMENT_PREFIX.length());
        }
        else 
        {
            throw new CommandException(getLocalizedString("CouldNotSetVariable",
                                                          new Object[] {name}));
        }
        return envName.toLowerCase();
    }

    
    /**
     * this method prints the environment variable.
     */
    private void printEnvValues()
    {
        if (commandEnvironment.getNumEnvironments() == 0)
            CLILogger.getInstance().printDetailMessage(getLocalizedString("NoEnvironment"));
        
        Iterator envIterator = commandEnvironment.getEnvironments().keySet().iterator();
        while (envIterator.hasNext())
        {
            final String name = (String) envIterator.next();
            final String envName = ENVIRONMENT_PREFIX.concat(name.toUpperCase());

            CLILogger.getInstance().printDebugMessage(commandEnvironment.toString());
            printOneEnvValue(name, envName);
        }
    }

    
    /**
     *  print one environment
     *  @param name used in the environment
     *  @param displayName is the display name with AS_ADMIN_ prefix
     */
    private void printOneEnvValue(String name, String displayName)
    {
        //Do not display the password value in clear text
        if (name.endsWith(PASSWORD_STRING))
        {
            CLILogger.getInstance().printMessage(displayName + " = ********");
        }
        else
        {
            CLILogger.getInstance().printMessage(displayName + " = " + 
                                                 (String)commandEnvironment.getEnvironmentValue(name));
        }
    }
    

    /** 
     *  this method updates the variable in the CommandEnvironment object.
     *  @param nameStr - the name of the environment 
     */
    private void updateEnvironment(String nameStr)
	throws CommandException
    {
        final int nameend = nameStr.indexOf(ENVIRONMENT_DELIMITER);

        if (nameend == nameStr.length()-1) 
        {
            final String envName = checkForPrefix(nameStr.substring(0, nameend));
            if (commandEnvironment.removeEnvironment(envName) == null)
                throw new CommandException(getLocalizedString("UnableToRemoveEnv",
                                                              new Object[] {nameStr}));
        }
        //print the value is the command is "export AS_ADMIN_<name>"
        else if (nameend == -1)
        {
            final String name = checkForPrefix(nameStr);
            printOneEnvValue(name, nameStr);
        }
        else 
        {
            final String envName = checkForPrefix(nameStr.substring(0, nameend));
            commandEnvironment.setEnvironment(envName,
                                              nameStr.substring(nameend+1));
        }
    }
}
