/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

/*
 * Copyright 2004-2005 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

package com.sun.enterprise.admin.monitor.stats;
import javax.management.j2ee.statistics.Stats;
import javax.management.j2ee.statistics.RangeStatistic;
import javax.management.j2ee.statistics.CountStatistic;
import com.sun.enterprise.admin.monitor.stats.AverageRangeStatistic;

/**
 *
 * @since 8.1
 */
public interface StatefulSessionStoreStats extends Stats {
    
    /**
     * Returns the number of passivated/checkpointed sessions in the
     * store
     * @return RangeStatistic
     */
    public RangeStatistic getCurrentSize();
    
    /**
     * Returns the total number of sessions activated from the store
     * @return CountStatistic
     */
    public CountStatistic getActivationCount();
    
    /**
     * Returns the total number of sessions successfully activated
     * from the store
     * @return CountStatistic
     */
    public CountStatistic getActivationSuccessCount();
    
    /**
     * Returns the total number of sessions that could not be activated
     * from the store
     * @return CountStatistic
     */
    public CountStatistic getActivationErrorCount();
    
    /**
     * Returns the total number of sessions passivated using this store
     * @return CountStatistic
     */
    public CountStatistic getPassivationCount();
    
    /**
     * Returns the total number of sessions passivated successfully
     * using this store
     * @return CountStatistic
     */
    public CountStatistic getPassivationSuccessCount();
    
    /**
     * Returns the total number of sessions that could not be passivated
     * using the store
     * @return CountStatistic
     */
    public CountStatistic getPassivationErrorCount();
    
    /**
     * Returns the total number of expired sessions that were removed by 
     * the store
     * @return CountStatistic
     */
    public CountStatistic getExpiredSessionCount();
    
    /**
     * Returns the total number of bytes passivated by this store
     * @return AverageRangeStatistic
     */
    public AverageRangeStatistic getPassivatedBeanSize();
    
    /**
     * Returns the time spent on passivating beans to the store
     * @return AverageRangeStatistic
     */
    public AverageRangeStatistic getPassivationTime();
    
    /**
     * Returns the total number of bytes activated by this store
     * @return AverageRangeStatistic
     */
    public AverageRangeStatistic getActivatedBeanSize();
    
    /**
     * Returns the time spent on activating beans from the store
     * @return AverageRangeStatistic
     */
    public AverageRangeStatistic getActivationTime();
    
}
