/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.enterprise.config.serverbeans.validation.tests;


import com.sun.enterprise.config.ConfigContextEvent;
import com.sun.enterprise.config.ConfigException;
import com.sun.enterprise.config.serverbeans.LifecycleModule;
import com.sun.enterprise.config.serverbeans.ServerBeansFactory;
import com.sun.enterprise.config.serverbeans.ServerTags;
import com.sun.enterprise.config.serverbeans.validation.Result;
import com.sun.enterprise.config.serverbeans.validation.ValidationDescriptor;
import com.sun.enterprise.config.serverbeans.validation.GenericValidator;
import java.util.Set;

/**
    Custom Test for LifecycleModule Element which calls the Generic Validation before performing custom tests

    @author Srinivas Krishnan
    @version 2.0
*/

public class LifecycleModuleTest extends GenericValidator {
    
    public LifecycleModuleTest(ValidationDescriptor desc) {
        super(desc);
    } 
    
    public Result validate(final ConfigContextEvent cce) {
        final Result result = super.validate(cce); // Before doing custom validation do basic validation
        if(cce.getChoice().equals(StaticTest.UPDATE) || cce.getChoice().equals(StaticTest.VALIDATE)) {
            validateAttribute(cce.getName(), (String) cce.getObject(), result);
        }
        return result;
    }

    private void validateAttribute(String name, String value, Result result) { 
        if(value == null || value.equals(""))
            return;
        if(name.equals(ServerTags.CLASSPATH)) {
            if(!StaticTest.isClassPathValid(value)) {
                result.failed(smh.getLocalString(getClass().getName() + ".invalidClasspath",
                   "Classpath contains invalid path : Check the path"));
            }
        }
    }
}
