/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */
package com.sun.enterprise.jbi.serviceengine.bridge;

import com.sun.enterprise.deployment.WebServiceEndpoint;
import com.sun.enterprise.webservice.monitoring.EndpointLifecycleListener;
import com.sun.enterprise.webservice.monitoring.Endpoint;
import com.sun.logging.LogDomains;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.namespace.QName;

/**
 *
 * @author Manisha Umbarje
 */
public class EndpointLifecycleListenerImpl implements EndpointLifecycleListener {
    
    /**
     * Internal handle to the logger instance
     */
    protected static final Logger logger =
        LogDomains.getLogger(LogDomains.SERVER_LOGGER);
    /**
     * Creates a new instance of EndpointLifecycleListenerImpl 
     */
    public EndpointLifecycleListenerImpl() {
    }
    
    
    /**
     * Notification of a new Web Service endpoint installation in the 
     * appserver. 
     * @param endpoint endpoint to register SOAPMessageListener if needed.
     */
    public void endpointAdded(Endpoint endpoint) {
            // End point added
        EndpointHelper.getInstance().registerEndpoint(endpoint);
    }
    
    /**
     * Notification of a Web Service endpoint removal from the appserver
     * @param endpoint handler to register SOAPMessageListener if needed.
     */
    public void endpointRemoved(Endpoint endpoint) {
        WebServiceEndpoint endpt = endpoint.getDescriptor();
        String endpointName = endpt.hasWsdlPort() ?
            endpt.getWsdlPort().getLocalPart() : endpt.getEndpointName();
        EndpointHelper.getInstance().unregisterEndpoint(
                endpt.getServiceName(), endpointName);
    }
}
