/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */
package com.sun.enterprise.deployment.annotation.handlers;

import java.lang.annotation.Annotation;
import java.lang.annotation.ElementType;
import java.lang.reflect.AnnotatedElement;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import javax.xml.ws.WebServiceRefs;
import javax.xml.ws.WebServiceRef;

import com.sun.enterprise.deployment.annotation.HandlerProcessingResult;
import com.sun.enterprise.deployment.annotation.AnnotationInfo;
import com.sun.enterprise.deployment.annotation.AnnotationProcessorException;
import com.sun.enterprise.deployment.annotation.HandlerProcessingResult;
import com.sun.enterprise.deployment.annotation.context.ResourceContainerContext;

/**
 * This handler is responsible for handling the WebServiceRefs annotation
 *
 */
public class WebServiceRefsHandler extends WebServiceRefHandler {
    
    public WebServiceRefsHandler() {
    }
    
    /**
     * @return the annoation type this annotation handler is handling
     */
    public Class<? extends Annotation> getAnnotationType() {
        return WebServiceRefs.class;
    }    
        

    public HandlerProcessingResult processAnnotation(AnnotationInfo ainfo)
            throws AnnotationProcessorException {

        WebServiceRefs wsRefsAnnotation = (WebServiceRefs) ainfo.getAnnotation();
        
        WebServiceRef[] wsRefAnnotations = wsRefsAnnotation.value();
        List<HandlerProcessingResult> results = new ArrayList<HandlerProcessingResult>();

        for(WebServiceRef wsRef : wsRefAnnotations) {
            results.add(processAWsRef(ainfo, wsRef));
        }
        HandlerProcessingResult finalResult = null;
        for (HandlerProcessingResult result : results) {
            if (finalResult == null ||
                    (result.getOverallResult().compareTo(
                    finalResult.getOverallResult()) > 0)) {
                finalResult = result;
            }
        }
        return finalResult;
    }
}
