/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

/*
 * RelationshipRoleSourceNode.java
 *
 * Created on February 1, 2002, 3:36 PM
 */

package com.sun.enterprise.deployment.node;

import java.util.Map;
import java.util.HashMap;
import com.sun.enterprise.deployment.Descriptor;
import com.sun.enterprise.deployment.xml.EjbTagNames;

/**
 * ConfigurableNode able to treat dispatch element values to descriptors based 
 * on initialized values
 *
 * @author  Jerome Dochez
 * @version 
 */
public class ConfigurableNode extends DeploymentDescriptorNode {

    private Map dispatchTable;
    private Object descriptor;
 
    public ConfigurableNode(Object instance, Map dispatchTable) {
        super();
        this.dispatchTable = dispatchTable;
        this.descriptor = instance;
    }

    public ConfigurableNode(Object descriptor, Map dispatchTable, XMLElement element) {
        super();
        this.dispatchTable = dispatchTable;
        this.descriptor = descriptor;
	super.setXMLRootTag(element);
    }

    /**
    * @return the descriptor instance to associate with this XMLNode
    */    
    public Object getDescriptor() {
        return descriptor;
    }        
    
    /**
     * all sub-implementation of this class can use a dispatch table to map xml element to
     * method name on the descriptor class for setting the element value. 
     *  
     * @return the map with the element name as a key, the setter method as a value
     */    
    protected Map getDispatchTable() {
        return dispatchTable;
    }    
}
