/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.enterprise.deployment.node;

import java.util.Map;
import java.util.Iterator;
import org.w3c.dom.Node;
import com.sun.enterprise.deployment.NameValuePairDescriptor;
import com.sun.enterprise.deployment.xml.TagNames;

/**
 * This class handles a name-value property 
 *
 * @author Jerome Dochez
 */
public class NameValuePairNode extends DeploymentDescriptorNode {
    
   /**
     * all sub-implementation of this class can use a dispatch table to map xml element to
     * method name on the descriptor class for setting the element value. 
     *  
     * @return the map with the element name as a key, the setter method as a value
     */    
    protected Map getDispatchTable() {    
        Map table = super.getDispatchTable();
        table.put(TagNames.NAME_VALUE_PAIR_NAME, "setName");
        table.put(TagNames.NAME_VALUE_PAIR_VALUE, "setValue");
        return table;
    }    
    
    /**
     * write the descriptor class to a DOM tree and return it
     *
     * @param parent node in the DOM tree 
     * @param node name for the root element of this xml fragment      
     * @param the descriptor to write
     * @return the DOM tree top node
     */
    public Node writeDescriptor(Node parent, String nodeName, NameValuePairDescriptor descriptor) {    
        Node envEntryNode = super.writeDescriptor(parent, nodeName, descriptor);
        appendTextChild(envEntryNode, TagNames.NAME_VALUE_PAIR_NAME, 
                        descriptor.getName());
        appendTextChild(envEntryNode, TagNames.NAME_VALUE_PAIR_VALUE, 
                        descriptor.getValue());        
        return envEntryNode;
    }
    
    /**
     * write the descriptor class to a DOM tree and return it
     *
     * @param parent node in the DOM tree 
     * @param node name for the root element of this xml fragment      
     * @param iterator on the descriptors to write
     * @return the DOM tree top node
     */
    public void writeDescriptor(Node parent, String nodeName, Iterator props) {
	if (props==null) 
	    return;
	    
	while(props.hasNext()) {
	    NameValuePairDescriptor aProp = (NameValuePairDescriptor) props.next();
	    writeDescriptor(parent, nodeName, aProp);
	}
    }
}
