/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.enterprise.deployment.node.web;

import java.util.Map;
import org.w3c.dom.Node;

import com.sun.enterprise.deployment.ErrorPageDescriptorImpl;
import com.sun.enterprise.deployment.xml.WebTagNames;
import com.sun.enterprise.deployment.node.DeploymentDescriptorNode;

/**
 * This node handles the error-page xml tag
 *
 * @author  Jerome Dochez
 * @version 
 */
public class ErrorPageNode extends DeploymentDescriptorNode {

    /**
     * all sub-implementation of this class can use a dispatch table to map xml element to
     * method name on the descriptor class for setting the element value. 
     *  
     * @return the map with the element name as a key, the setter method as a value
     */    
    protected Map getDispatchTable() {    
        Map table = super.getDispatchTable();
        table.put(WebTagNames.ERROR_CODE, "setErrorSignifierAsString");
        table.put(WebTagNames.EXCEPTION_TYPE, "setExceptionType");
        table.put(WebTagNames.LOCATION, "setLocation");        
        return table;
    }    
    
    /**
     * write the descriptor class to a DOM tree and return it
     *
     * @param parent node in the DOM tree 
     * @param node name for the root element of this xml fragment      
     * @param the descriptor to write
     * @return the DOM tree top node
     */
    public Node writeDescriptor(Node parent, String nodeName, ErrorPageDescriptorImpl descriptor) {       
        Node myNode = appendChild(parent, nodeName);    
        String exceptionType = descriptor.getExceptionType();
        if (exceptionType!=null && exceptionType.length()!=0) {
            appendTextChild(myNode, WebTagNames.EXCEPTION_TYPE, exceptionType);
        } else {
            appendTextChild(myNode, WebTagNames.ERROR_CODE, descriptor.getErrorSignifierAsString());
        }
        appendTextChild(myNode, WebTagNames.LOCATION, descriptor.getLocation());
        return myNode;
    }
}
