/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

/**
 * <BR> <I>$Source: /cvs/glassfish/appserv-core/src/java/com/sun/ejb/containers/util/pool/BoundedPool.java,v $</I>
 * @author     $Author: tcfujii $
 * @version    $Revision: 1.3 $ $Date: 2005/12/25 04:13:35 $
 */
 

package com.sun.ejb.containers.util.pool;

import java.util.Collection;
import java.util.ArrayList;
import java.util.logging.*;

import com.sun.logging.*;

/**
 * <p>Abstract pool provides the basic implementation of an object pool. 
 * The implementation uses a linked list to maintain a list of (available) 
 * objects. If the pool is empty it simply creates one using the 
 * ObjectFactory instance. Subclasses can change this behaviour by overriding 
 * getObject(...) and returnObject(....) methods. This class provides basic 
 * support for synchronization, event notification, pool shutdown and pool 
 * object recycling. It also does some very basic bookkeeping like the
 * number of objects created, number of threads waiting for object.
 * <p> Subclasses can make use of these book-keeping data to provide complex 
 * pooling mechanism like LRU / MRU / Random. Also, note that AbstractPool 
 * does not have a notion of  pool limit. It is upto to the derived classes 
 * to implement these features.
 *	
 */
public class BoundedPool
    extends AbstractPool
{
    
    private static Logger _logger;
    static{
        _logger=LogDomains.getLogger(LogDomains.EJB_LOGGER);
    }
    
    protected int previousSize = 0;
    
    public BoundedPool(ObjectFactory factory, int steadyPoolSize, 
        int resizeQuantity, int maxPoolsize, long maxWaitTimeInMillis, 
        int idleTimeoutInSeconds, ClassLoader loader)
    {
    	super(factory, steadyPoolSize, resizeQuantity, maxPoolsize,
              maxWaitTimeInMillis, idleTimeoutInSeconds, loader);
        super.poolName="BoundedPool";
    }
    
    protected void removeIdleObjects() {
        int curSize = 0;
        int count = 0;
        synchronized (list) {
            curSize = list.size();
        }

        if(curSize <= steadyPoolSize)
            return; // no need to trim the pool beyond steadyPoolSize
        count = (curSize > (steadyPoolSize + resizeQuantity) ) ? 
            resizeQuantity: (curSize - steadyPoolSize);
        previousSize = curSize;
        
        if (count > 0) {
            _logger.log(Level.FINE,
                        "BoundedPool removing " + count + " objects");
            super.remove(count);
        }
    }

}
