/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

/*
 * DisassociationPhase.java
 *
 * Created on May 20, 2003, 3:17 PM
 * @author  sandhyae
 * <BR> <I>$Source: /cvs/glassfish/appserv-core/src/java/com/sun/enterprise/deployment/phasing/DisassociationPhase.java,v $
 *
 */

package com.sun.enterprise.deployment.phasing;

import com.sun.enterprise.deployment.backend.IASDeploymentException;
import com.sun.enterprise.deployment.backend.DeploymentRequest;
import com.sun.enterprise.deployment.backend.DeploymentEvent;
import com.sun.enterprise.deployment.backend.DeploymentEventType;
import com.sun.enterprise.deployment.backend.DeploymentEventInfo;
import com.sun.enterprise.deployment.backend.DeploymentLogger;
import com.sun.enterprise.deployment.backend.DeploymentStatus;
import com.sun.enterprise.util.i18n.StringManager;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This phase is responsible to disassociate an application from a target
 * It uses ServerDeploymentTarget, GroupDeploymentTarget to remove references
 * @author  Sandhya E
 */
class DisassociationPhase extends DeploymentPhase {
   
    /** Deployment Logger object for this class */
    public static final Logger sLogger = DeploymentLogger.get();    
    
    /** string manager */
    private static StringManager localStrings =
        StringManager.getManager( DisassociationPhase.class );
    
    /** 
     * Creates a new instance of DisassociatePhase 
     * @param deploymentCtx DeploymentContext object
     */
    public DisassociationPhase(DeploymentContext deploymentCtx) {
        this.deploymentCtx = deploymentCtx;
        this.name = DISASSOCIATE;        
    }
    
    /**
     * This method disassociates the application from the target as specified 
     * in request object     
     * @param req DeploymentRequest object
     * @param phaseCtx the DeploymentPhaseContext object     
     */
    public void runPhase(DeploymentPhaseContext phaseCtx)
    {
        DeploymentStatus status = phaseCtx.getDeploymentStatus();

        try {
            //get the app directory on target's cache deleted
            DeploymentRequest req = phaseCtx.getDeploymentRequest();
            DeploymentTarget target = (DeploymentTarget)req.getTarget();

            target.removeAppReference(req.getName());
            sendDisassociateEvent(req);
            status.setStageStatus(DeploymentStatus.SUCCESS);
        }catch(Throwable t){
            status.setStageStatus(DeploymentStatus.FAILURE);
            status.setStageException(t);
            status.setStageStatusMessage(t.getMessage());
        }
    }
 
    
    /**
     * Event that will be broadcasted at the start of the phase
     * @param req DeploymentRequest object
     * @return DeploymentEvent
     **/
    private DeploymentEvent getPrePhaseEvent(DeploymentRequest req) {
        return new DeploymentEvent(DeploymentEventType.PRE_DISASSOCIATE, new DeploymentEventInfo(req));
    }
    
    /**
     * Event that will be broadcasted at the end of the phase
     * @param req DeploymentRequest object
     * @return DeploymentEvent
     **/
    private DeploymentEvent getPostPhaseEvent(DeploymentRequest req) {
        return new DeploymentEvent(DeploymentEventType.POST_DISASSOCIATE,new DeploymentEventInfo(req));
    }

    private void sendDisassociateEvent(DeploymentRequest req) throws 
        Exception {
        DeploymentTarget target = (DeploymentTarget)req.getTarget();

        String moduleType;

        if(req.isApplication()) {
            moduleType = null;
        }
        else {
            moduleType = DeploymentServiceUtils.getModuleTypeString(req.getType());
        }

        int eventType = com.sun.enterprise.admin.event.BaseDeployEvent.APPLICATION_UNREFERENCED;
        String appName = req.getName();
        String targetName = target.getTarget().getName();

        boolean success = DeploymentServiceUtils.multicastEvent(
                          eventType,
                          appName,
                          moduleType,
                          false,
                          true, 
                          targetName);
 
        sLogger.log(Level.FINE, "sendDisassociateEvent: success=" + success);
    }   
}
