/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

/*
 * EJBSecurityManagerFactory.java
 *
 * Created on June 9, 2003, 5:42 PM
 */

package com.sun.enterprise.security.factory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.sun.enterprise.SecurityManager;
import com.sun.enterprise.deployment.Descriptor;
import com.sun.enterprise.deployment.EjbDescriptor;
import com.sun.enterprise.security.application.EJBSecurityManager;

import java.util.logging.Level;
/**
 * EJB Security Manager Factory Implementation
 * @author  Harpreet Singh
 */
public final class EJBSecurityManagerFactory
    extends AbstractSecurityManagerFactory {
    private Map CONTEXT_ID = new HashMap();

    /** Creates a new instance of EJBSecurityManagerFactory */
    private EJBSecurityManagerFactory() {
    }
    
    public static SecurityManagerFactory getInstance() {
        if(_theFactory == null){
            _theFactory = (SecurityManagerFactory)new EJBSecurityManagerFactory();
        }
        return  _theFactory;
    }

    public SecurityManager getSecurityManager(String contextId){            
        if (_poolHas(contextId)){
            return (SecurityManager)_poolGet(contextId);
        }
        return null;
    }        
    
    public SecurityManager createSecurityManager(Descriptor descriptor) {
        SecurityManager ejbSM = null;
        String contextId = null;
        String appName = null;
        try {
            ejbSM = EJBSecurityManager.getInstance(descriptor);
            // if the descriptor is not a EjbDescriptor the EJBSM will 
            // throw an exception. So the following will always work.
            EjbDescriptor ejbdes = (EjbDescriptor) descriptor;
            appName = ejbdes.getApplication().getRegistrationName();
            contextId = EJBSecurityManager.getContextID(ejbdes);
            if(_logger.isLoggable(Level.FINE)){
                _logger.log(Level.FINE,
                "[EJB-Security] EJB Security:Creating EJBSecurityManager for contextId = "
                +contextId);
            }

        } catch (Exception e){
            _logger.log(Level.FINE,
            "[EJB-Security] FATAl Exception. Unable to create EJBSecurityManager: "
            + e.getMessage() );
            throw new RuntimeException(e);
        } 

	synchronized (CONTEXT_ID) {
	    List lst = (List)CONTEXT_ID.get(appName);
	    if(lst == null){
		lst = new ArrayList();
		CONTEXT_ID.put(appName, lst);
	    }
	    if (!lst.contains(contextId)) {
		lst.add(contextId);
	    }
	}

        _poolPut(contextId, ejbSM);
        return ejbSM;     
    }

    public String[] getAndRemoveContextIdForEjbAppName(String appName){
	synchronized(CONTEXT_ID) {
            List contextId = (List) CONTEXT_ID.get(appName);
            if (contextId == null) {
                return null;
            }
            String[] rvalue = new String[contextId.size()];
            rvalue = (String[])contextId.toArray(rvalue);

	    CONTEXT_ID.remove(appName);
	    return rvalue;
	}
    }
}
