/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */

package com.sun.enterprise.server;

import com.sun.enterprise.admin.event.AdminEventListener;
import com.sun.enterprise.admin.event.jms.*;
import com.sun.enterprise.admin.event.AdminEventListenerException;
import com.sun.enterprise.config.serverbeans.*;
import com.sun.enterprise.config.*;
import com.sun.enterprise.connectors.*;
import com.sun.enterprise.connectors.system.*;


/**
 * Listener interface to handle jms-service element events.
 *
 * @author: alexkrav
 */
public class JmsServiceEventListener implements 
       com.sun.enterprise.admin.event.jms.JmsServiceEventListener {

    private ConnectorRegistry registry = ConnectorRegistry.getInstance();

    /**
     * Handles jms-service element removal.
     * It is called whenever a JmsServiceEvent is received.
     *
     * @param event                        Event to be processed.
     *
     * @throws AdminEventListenerException when the listener is unable to
     *         process the event.
     */
    public void handleDelete(JmsServiceEvent event)
             throws AdminEventListenerException {
    }

    /**
     * Handles jms-service element modification 
     * (attributes/properties values changed).
     * It is called whenever a JmsServiceEvent is received.
     *
     * @param event                        Event to be processed.
     *
     * @throws AdminEventListenerException when the listener is unable to
     *         process the event.
     */
    public void handleUpdate(JmsServiceEvent event)
             throws AdminEventListenerException {
        try {
            JmsService service = getJmsService(event);
            ActiveJmsResourceAdapter ajr = (ActiveJmsResourceAdapter) 
                registry.getActiveResourceAdapter(ConnectorRuntime.DEFAULT_JMS_ADAPTER);
            if (ajr != null) {
                ajr.reloadRA(service);
            }
            JmsService oldService =  ServerBeansFactory.getJmsServiceBean(
                event.getOldConfigContext());
            if(!oldService.getType().equals(service.getType()))
                throw new AdminEventListenerException
                ("jms type is not dynamically configured");    
        } catch (ConnectorRuntimeException cre) {
            AdminEventListenerException ale = 
                new AdminEventListenerException(cre.getMessage());
            ale.initCause(cre);
            throw ale;
        } catch (Exception ce) {
            AdminEventListenerException ale = 
                new AdminEventListenerException(ce.getMessage());
            ale.initCause(ce);
            throw ale;
        }
    }

    /**
     * Handles element additions.
     * It is called whenever a JmsServiceEvent is received.
     *
     * @param event                        Event to be processed.
     *
     * @throws AdminEventListenerException when the listener is unable to
     *         process the event.
     */
    public void handleCreate(JmsServiceEvent event)
             throws AdminEventListenerException {
    }

    private JmsService getJmsService(JmsServiceEvent event) throws ConfigException {
        ConfigContext context = event.getConfigContext();
        return ServerBeansFactory.getJmsServiceBean(context);
    }
}
