/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */
package com.sun.enterprise.tools.verifier.tests.ejb.homeintf;

import com.sun.enterprise.deployment.EjbDescriptor;
import com.sun.enterprise.tools.verifier.Result;
import com.sun.enterprise.tools.verifier.Verifier;
import com.sun.enterprise.tools.verifier.tests.ComponentNameConstructor;

import java.lang.ClassLoader;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

/** 
 * Home methods must be public and not static
 *
 * @author Jerome Dochez
 * @version
 */
abstract public class HomeMethodModifiers extends HomeMethodTest {  
    
    /**
     * <p>
     * run an individual home method test 
     * </p>
     * 
     * @param method the home method to test
     * @param descriptor the deployment descriptor for the entity bean
     * @param result the result object
     */

    protected void runIndividualHomeMethodTest(Method method, EjbDescriptor descriptor, Result result) {
        
        Method m;
	ComponentNameConstructor compName = getVerifierContext().getComponentNameConstructor();

	try {	  
	    // retrieve the remote interface methods
	    ClassLoader jcl = getVerifierContext().getClassLoader();
	    Class ejbClass = Class.forName(descriptor.getEjbClassName(), false, jcl);
                                    
            // Bug: 4952890. first character of this name should be converted to UpperCase. 
            String methodName = method.getName().replaceFirst(method.getName().substring(0,1),
                                                              method.getName().substring(0,1).toUpperCase());
            String expectedMethodName = "ejbHome" + methodName;
            do {
                // retrieve the EJB Class Methods
                m = getMethod(ejbClass, expectedMethodName, method.getParameterTypes());   
	    } while (((ejbClass = ejbClass.getSuperclass()) != null) && (m==null));

            if (m != null) {
                int modifiers = m.getModifiers();
                if (Modifier.isPublic(modifiers) && !Modifier.isStatic(modifiers)) {
                    addGoodDetails(result, compName);
                    result.addGoodDetails(smh.getLocalString
                        (getClass().getName() + ".passed",
                        "For method [ {1} ] in Home Interface [ {0} ], ejbHome method is public and not static",
                        new Object[] {method.getDeclaringClass().getName(), method.getName()})); 
		    result.setStatus(Result.PASSED);               
                    //return true;
                } else {
                    addErrorDetails(result, compName);
                    result.addErrorDetails(smh.getLocalString
                        (getClass().getName() + ".notApplicable",
                        "Error : For method [ {1} ] defined in Home Interface [ {0} ], the ejbHome method is either static or not public",
                        new Object[] {method.getDeclaringClass().getName(), method.getName()}));
		    result.setStatus(Result.FAILED);               
		    // return false;
                }                    
            } else {
                addErrorDetails(result, compName);
                result.addErrorDetails(smh.getLocalString
                    (getClass().getName() + ".failed",  
                    "Error : For method [ {1} ] defined in Home Interface [ {0} ], no ejbHome name matching method was found" ,
                    new Object[] {method.getDeclaringClass().getName(), method.getName()}));
		    result.setStatus(Result.FAILED);               
                //return true;
	    }
	} catch (ClassNotFoundException e) {
	    Verifier.debug(e);
        addErrorDetails(result, compName);
	    result.failed(smh.getLocalString(
					     getClass().getName() + ".failedException",
					     "Error: Home interface [ {0} ] does not exist or is not loadable within bean [ {1} ]",
					     new Object[] {getClassName(descriptor),descriptor.getName()}));
	    //return false;
	    result.setStatus(Result.FAILED);
	}
    }

  private String getClassName(EjbDescriptor descriptor) {
	return getHomeInterfaceName(descriptor);
    }  
}
