/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */
package com.sun.enterprise.tools.verifier.tests.ejb.intf;

import com.sun.enterprise.deployment.EjbDescriptor;
import com.sun.enterprise.deployment.EjbEntityDescriptor;
import com.sun.enterprise.deployment.EjbSessionDescriptor;
import com.sun.enterprise.tools.verifier.Result;
import com.sun.enterprise.tools.verifier.tests.ComponentNameConstructor;
import com.sun.enterprise.tools.verifier.tests.ejb.EjbCheck;
import com.sun.enterprise.tools.verifier.tests.ejb.EjbTest;

import java.lang.reflect.Modifier;
import java.util.Set;
import java.util.logging.Level;

/** 
 * Declare local and remote interfaces as public interfaces test.  
 * All enterprise bean local and/or remote interfaces must be declared as public.
 */
abstract public class InterfacePublic extends EjbTest implements EjbCheck { 
    
    /**
     * Methods to get the type of interface: local/remote and the name of the class
     */
    
    abstract protected Set<String> getInterfaceNames(EjbDescriptor descriptor);
    
    /** 
     * Declare local and remote interfaces as public interfaces test.  
     * All enterprise bean local and/or interfaces must be declared as public.
     *   
     * @param descriptor the Enterprise Java Bean deployment descriptor
     *   
     * @return <code>Result</code> the results for this assertion
     */
    public Result check(EjbDescriptor descriptor) {
        Result result = getInitializedResult();
        ComponentNameConstructor compName = getVerifierContext().getComponentNameConstructor();
        
        if (!(descriptor instanceof EjbSessionDescriptor) &&
                !(descriptor instanceof EjbEntityDescriptor)) { 
            addNaDetails(result, compName);
            result.notApplicable(smh.getLocalString
                    ("com.sun.enterprise.tools.verifier.tests.ejb.homeintf.HomeMethodTest.notApplicable1",
                    "Test apply only to session or entity beans."));
            return result;                
        }
        String assertionClass = "com.sun.enterprise.tools.verifier.tests.ejb.intf.InterfacePublic";

        for (String intfName : getInterfaceNames(descriptor)) {
            try {
                ClassLoader jcl = getVerifierContext().getClassLoader();
                Class c = Class.forName(intfName, false, jcl);
                
                // local and remote interface must be defined as public
                if (!Modifier.isPublic(c.getModifiers())) {
                    addErrorDetails(result, compName);
                    result.failed(smh.getLocalString
                            (assertionClass + ".failed",
                            "Error: [ {0} ] is not defined as a public interface.",
                            new Object[] {intfName}));
                }
            } catch (ClassNotFoundException e) {
                // ignore as it will be caught in EjbArchiveClassesLoadable
                logger.log(Level.FINER,e.getMessage(),e);
            }
        }
        
        if(result.getStatus() != Result.FAILED) {
            addGoodDetails(result, compName);
            result.passed(smh.getLocalString
                            (assertionClass + ".passed",
                            "Valid public interface(s)."));
        }
        return result;
    }
}
